@@ -25,6 +25,7 @@
 #include <string.h>
 #include <stdlib.h>
 #include <errno.h>
+#include <limits.h>
 #include <regex.h>
 #include <sys/time.h>
 #include "echo_loader.h"
@@ -123,9 +124,7 @@ static void usb_buffer_showstatistics(struct astribank_device *astribank, struct
 	long	usec;
 
 	usec = usb_buffer_usec(ub);
-	INFO("%s [%s]: Octasic statistics: packet_size=[%d, %ld, %d] packets=%d, bytes=%ld msec=%ld usec/packet=%d\n",
-		xusb_devpath(astribank->xusb),
-		xusb_serial(astribank->xusb),
+	AB_INFO(astribank, "Octasic statistics: packet_size=[%d, %ld, %d] packets=%d, bytes=%ld msec=%ld usec/packet=%ld\n",
 		ub->min_send,
 		ub->total_bytes / ub->num_sends,
 		ub->max_send,
@@ -143,8 +142,8 @@ static int usb_buffer_flush(struct astribank_device *astribank, struct usb_buffe
 	if (ub->curr == 0)
 		return 0;
 	ret = xusb_send(astribank->xusb, ub->data, ub->curr, TIMEOUT);
-	if(ret < 0) {
-		ERR("xusb_send failed: %d\n", ret);
+	if (ret < 0) {
+		AB_ERR(astribank, "xusb_send failed: %d\n", ret);
 		return ret;
 	}
 	DBG("%s: Written %d bytes\n", __func__, ret);
@@ -179,7 +178,7 @@ static int usb_buffer_append(struct astribank_device *astribank, struct usb_buff
 	char *buf, int len)
 {
 	if (ub->curr + len >= ub->max_len) {
-		ERR("%s: buffer too small ub->curr=%d, len=%d, ub->max_len=%d\n",
+		AB_ERR(astribank, "%s: buffer too small ub->curr=%d, len=%d, ub->max_len=%d\n",
 			__func__, ub->curr, len, ub->max_len);
 		return -ENOMEM;
 	}
@@ -210,15 +209,18 @@ static int usb_buffer_send(struct astribank_device *astribank, struct usb_buffer
 		if (ret < 0)
 			return ret;
 		ret = xusb_recv(astribank->xusb, buf, PACKET_SIZE, TIMEOUT);
-		if(ret <= 0) {
-			ERR("No USB packs to read: %s\n", strerror(-ret));
+		if (ret <= 0) {
+			AB_ERR(astribank, "No USB packs to read: %s\n", strerror(-ret));
 			return -EINVAL;
 		}
 		DBG("%s: %d bytes recv\n", __func__, ret);
 		phead = (struct xpp_packet_header *)buf;
-		if(phead->header.op != SPI_RCV_XOP && phead->header.op != TST_RCV_XOP) {
-			ERR("Got unexpected reply OP=0x%02X\n", phead->header.op);
-			dump_packet(LOG_ERR, DBG_MASK, "hexline[ERR]", buf, ret);
+		if (phead->header.op != SPI_RCV_XOP &&
+			phead->header.op != TST_RCV_XOP) {
+			AB_ERR(astribank, "Got unexpected reply OP=0x%02X\n",
+				phead->header.op);
+			dump_packet(LOG_ERR, DBG_MASK, "hexline[ERR]",
+				buf, ret);
 			return -EINVAL;
 		}
 		dump_packet(LOG_DEBUG, DBG_MASK, "dump:echoline[R]", (char *)phead, phead->header.len);
@@ -242,14 +244,16 @@ int spi_send(struct astribank_device *astribank, uint16_t addr, uint16_t data, i
 	char				buf[PACKET_SIZE];
 	struct xpp_packet_header	*phead = (struct xpp_packet_header *)buf;
 	int				pack_len;
+	int				spi_flags;
 
 	assert(astribank != NULL);
+	spi_flags = 0x30 | (recv_answer ? 0x40 : 0x00) | (ver ? 0x01 : 0x00);
 	pack_len = sizeof(phead->header) + sizeof(phead->alt.spi_pack);
 	phead->header.len 		= pack_len;
 	phead->header.op 		= SPI_SND_XOP;
 	phead->header.unit 		= 0x40;	/* EC has always this unit num */
-	phead->alt.spi_pack.header 	= 0x05; 
-	phead->alt.spi_pack.flags 	= 0x30 | (recv_answer ? 0x40: 0x00) | (ver ? 0x01: 0x00); 
+	phead->alt.spi_pack.header 	= 0x05;
+	phead->alt.spi_pack.flags 	= spi_flags;
 	phead->alt.spi_pack.addr_l	= (addr >> 0) & 0xFF;
 	phead->alt.spi_pack.addr_h	= (addr >> 8) & 0xFF;
 	phead->alt.spi_pack.data_l	= (data >> 0) & 0xFF;
@@ -259,8 +263,8 @@ int spi_send(struct astribank_device *astribank, uint16_t addr, uint16_t data, i
 
 
 	ret = usb_buffer_send(astribank, &usb_buffer, buf, pack_len, TIMEOUT, recv_answer);
-	if(ret < 0) {
-		ERR("usb_buffer_send failed: %d\n", ret);
+	if (ret < 0) {
+		AB_ERR(astribank, "usb_buffer_send failed: %d\n", ret);
 		return ret;
 	}
 	DBG("%s: Written %d bytes\n", __func__, ret);
@@ -269,33 +273,35 @@ int spi_send(struct astribank_device *astribank, uint16_t addr, uint16_t data, i
 
 int test_send(struct astribank_device *astribank)
 {
-        int                             ret;
-        char                            buf[PACKET_SIZE];
-        struct xpp_packet_header       *phead = (struct xpp_packet_header *)buf;
-        int                             pack_len;
+	int                             ret;
+	char                            buf[PACKET_SIZE];
+	struct xpp_packet_header       *phead = (struct xpp_packet_header *)buf;
+	int                             pack_len;
 
-        assert(astribank != NULL);
-        pack_len = sizeof(phead->header) + sizeof(phead->alt.tst_pack);
-        phead->header.len               = 6;
-        phead->header.op                = 0x35;
-        phead->header.unit              = 0x00;
-        phead->alt.tst_pack.tid         = 0x28; // EC TestId
-        phead->alt.tst_pack.tsid        = 0x00; // EC SubId
+	assert(astribank != NULL);
+	pack_len = sizeof(phead->header) + sizeof(phead->alt.tst_pack);
+	phead->header.len               = 6;
+	phead->header.op                = 0x35;
+	phead->header.unit              = 0x00;
+	phead->alt.tst_pack.tid         = 0x28; /* EC TestId	*/
+	phead->alt.tst_pack.tsid        = 0x00; /* EC SubId	*/
 
-        dump_packet(LOG_DEBUG, DBG_MASK, "dump:echoline[W]", (char *)phead, pack_len);
+	dump_packet(LOG_DEBUG, DBG_MASK, "dump:echoline[W]",
+		(char *)phead, pack_len);
 
-
-        ret = usb_buffer_send(astribank, &usb_buffer, buf, pack_len, TIMEOUT, 1);
-        if(ret < 0) {
-                ERR("usb_buffer_send failed: %d\n", ret);
-                return ret;
-        }
-        DBG("%s: Written %d bytes\n", __func__, ret);
-        return ret;
+	ret = usb_buffer_send(astribank,
+		&usb_buffer, buf, pack_len, TIMEOUT, 1);
+	if (ret < 0) {
+		AB_ERR(astribank, "usb_buffer_send failed: %d\n", ret);
+		return ret;
+	}
+	DBG("%s: Written %d bytes\n", __func__, ret);
+	return ret;
 }
 
-void echo_send_data(struct astribank_device *astribank, const unsigned int addr, const unsigned int data)
+int echo_send_data(struct astribank_device *astribank, const unsigned int addr, const unsigned int data)
 {
+	int ret;
 /*	DBG("SEND: %04X -> [%04X]\n", data, addr);
 	DBG("\t\t[%04X] <- %04X\n", 0x0008, (addr >> 20));
 	DBG("\t\t[%04X] <- %04X\n", 0x000A, (addr >> 4) & ((1 << 16) - 1));
@@ -304,55 +310,79 @@ void echo_send_data(struct astribank_device *astribank, const unsigned int addr,
  */
 
 	DBG("SND:\n");
-	spi_send(astribank, 0x0008, (addr >> 20)			, 0, 0);
-	spi_send(astribank, 0x000A, (addr >> 4) & ((1 << 16) - 1)	, 0, 0);
-	spi_send(astribank, 0x0004, data				, 0, 0);
-	spi_send(astribank, 0x0000, (((addr >> 1) & 0x7) << 9) | 
+	ret = spi_send(astribank, 0x0008, (addr >> 20)			, 0, 0);
+	if (ret < 0)
+		goto failed;
+	ret = spi_send(astribank, 0x000A, (addr >> 4) & ((1 << 16) - 1)	, 0, 0);
+	if (ret < 0)
+		goto failed;
+	ret = spi_send(astribank, 0x0004, data				, 0, 0);
+	if (ret < 0)
+		goto failed;
+	ret = spi_send(astribank, 0x0000, (((addr >> 1) & 0x7) << 9) |
 				(1 << 8) | (3 << 12) | 1		, 0, 0);
+	if (ret < 0)
+		goto failed;
+	return cOCT6100_ERR_OK;
+failed:
+	AB_ERR(astribank, "echo_send_data: spi_send failed (ret = %d)\n", ret);
+	return ret;
 }
 
-unsigned int echo_recv_data(struct astribank_device *astribank, const unsigned int addr)
+int echo_recv_data(struct astribank_device *astribank, const unsigned int addr)
 {
 	unsigned int data = 0x00;
-	unsigned int ret;
+	int ret;
 
 	DBG("RCV:\n");
-	spi_send(astribank, 0x0008, (addr >> 20)			, 0, 0);
-	spi_send(astribank, 0x000A, (addr >> 4) & ((1 << 16) - 1)	, 0, 0);
-	spi_send(astribank, 0x0000, (((addr >> 1) & 0x7) << 9) | 
+	ret = spi_send(astribank, 0x0008, (addr >> 20)			, 0, 0);
+	if (ret < 0)
+		goto failed;
+	ret = spi_send(astribank, 0x000A, (addr >> 4) & ((1 << 16) - 1)	, 0, 0);
+	if (ret < 0)
+		goto failed;
+	ret = spi_send(astribank, 0x0000, (((addr >> 1) & 0x7) << 9) |
 				(1 << 8) | 1				, 0, 0);
+	if (ret < 0)
+		goto failed;
 	ret = spi_send(astribank, 0x0004, data				, 1, 0);
-	return ret; 
+	if (ret < 0)
+		goto failed;
+	return ret;
+failed:
+	AB_ERR(astribank, "echo_recv_data: spi_send failed (ret = %d)\n", ret);
+	return ret;
 }
 
 int load_file(char *filename, unsigned char **ppBuf, UINT32 *pLen)
 {
-	unsigned char * pbyFileData = NULL;
-	FILE* pFile; 
+	unsigned char *pbyFileData = NULL;
+	FILE *pFile;
 
 	DBG("Loading %s file...\n", filename);
-	pFile = fopen( filename, "rb" ); 
-	if (pFile == NULL) { 
-		ERR("fopen\n");  
+	pFile = fopen(filename, "rb");
+	if (pFile == NULL) {
+		ERR("fopen\n");
 		return -ENODEV;
-	} 
+	}
 
-	fseek( pFile, 0L, SEEK_END ); 
-	*pLen = ftell( pFile ); 
-	fseek( pFile, 0L, SEEK_SET ); 
+	fseek(pFile, 0L, SEEK_END);
+	*pLen = ftell(pFile);
+	fseek(pFile, 0L, SEEK_SET);
 
-	pbyFileData = (unsigned char *)malloc(*pLen); 
-	if (pbyFileData == NULL) { 
-		fclose( pFile ); 
-		ERR("malloc\n" );  
+	pbyFileData = (unsigned char *)malloc(*pLen);
+	if (pbyFileData == NULL) {
+		fclose(pFile);
+		ERR("malloc\n");
 		return -ENODEV;
 	} else {
 		DBG("allocated mem for pbyFileData\n");
 	}
-	fread(pbyFileData, 1, *pLen, pFile); 
-	fclose(pFile); 
-	DBG("Successful loading %s file into memory (size = %d, DUMP: first = %02X %02X, last = %02X %02X)\n", 
-			filename, *pLen, 
+	fread(pbyFileData, 1, *pLen, pFile);
+	fclose(pFile);
+	DBG("Successful loading %s file into memory "
+		"(size = %d, DUMP: first = %02X %02X, last = %02X %02X)\n",
+			filename, *pLen,
 			pbyFileData[0], pbyFileData[1],
 			pbyFileData[(*pLen)-2], pbyFileData[(*pLen)-1]);
 	*ppBuf = pbyFileData;
@@ -367,7 +397,7 @@ UINT32 Oct6100UserGetTime(tPOCT6100_GET_TIME f_pTime)
 	unsigned int mask = ~0;
 	
 	gettimeofday(&tv, 0);
-	total_usecs = (((unsigned long long)(tv.tv_sec)) * 1000000) + 
+	total_usecs = (((unsigned long long)(tv.tv_sec)) * 1000000) +
 				  (((unsigned long long)(tv.tv_usec)));
 	f_pTime->aulWallTimeUs[0] = (total_usecs & mask);
 	f_pTime->aulWallTimeUs[1] = (total_usecs >> 32);
@@ -418,75 +448,111 @@ UINT32 Oct6100UserDriverWriteApi(tPOCT6100_WRITE_PARAMS f_pWriteParams)
 	const unsigned int 		data 		= f_pWriteParams->usWriteData;
 	const struct echo_mod		*echo_mod 	= (struct echo_mod *)(f_pWriteParams->pProcessContext);
 	struct astribank_device 	*astribank 	= echo_mod->astribank;
+	int ret;
 
-	echo_send_data(astribank, addr, data);
-
+	ret = echo_send_data(astribank, addr, data);
+	if (ret < 0) {
+		ERR("echo_send_data failed (ret = %d)\n", ret);
+		return cOCT6100_ERR_FATAL_DRIVER_WRITE_API;
+	}
 	return cOCT6100_ERR_OK;
 }
 
 UINT32 Oct6100UserDriverWriteSmearApi(tPOCT6100_WRITE_SMEAR_PARAMS f_pSmearParams)
 {
-        unsigned int              	addr;
-        unsigned int              	data;
-        unsigned int              	len		= f_pSmearParams->ulWriteLength;
-        const struct echo_mod           *echo_mod       = (struct echo_mod *)f_pSmearParams->pProcessContext;
-        struct astribank_device   	*astribank      = echo_mod->astribank;
+	unsigned int              	addr;
+	unsigned int              	data;
+	unsigned int              	len;
+	const struct echo_mod           *echo_mod;
+	struct astribank_device   	*astribank;
 	unsigned int 			i;
 
+	len = f_pSmearParams->ulWriteLength;
+	echo_mod = (struct echo_mod *)f_pSmearParams->pProcessContext;
+	astribank = echo_mod->astribank;
 	for (i = 0; i < len; i++) {
+		int ret;
+
 		addr = f_pSmearParams->ulWriteAddress + (i << 1);
 		data = f_pSmearParams->usWriteData;
-		echo_send_data(astribank, addr, data);
+		ret = echo_send_data(astribank, addr, data);
+		if (ret < 0) {
+			ERR("echo_send_data failed (ret = %d)\n", ret);
+			return cOCT6100_ERR_FATAL_DRIVER_WRITE_API;
+		}
 	}
 	return cOCT6100_ERR_OK;
 }
 
 UINT32 Oct6100UserDriverWriteBurstApi(tPOCT6100_WRITE_BURST_PARAMS f_pBurstParams)
 {
-        unsigned int              	addr;
-        unsigned int              	data;
+	unsigned int              	addr;
+	unsigned int              	data;
 	unsigned int 			len 		= f_pBurstParams->ulWriteLength;
 	const struct echo_mod		*echo_mod 	= (struct echo_mod *)f_pBurstParams->pProcessContext;
 	struct astribank_device 	*astribank 	= echo_mod->astribank;
 	unsigned int 			i;
 
 	for (i = 0; i < len; i++) {
+		int ret;
+
 		addr = f_pBurstParams->ulWriteAddress + (i << 1);
 		data = f_pBurstParams->pusWriteData[i];
-		echo_send_data(astribank, addr, data);
+		ret = echo_send_data(astribank, addr, data);
+		if (ret < 0) {
+			ERR("echo_send_data failed (ret = %d)\n", ret);
+			return cOCT6100_ERR_FATAL_DRIVER_WRITE_API;
+		}
 	}
 	return cOCT6100_ERR_OK;
 }
 
 UINT32 Oct6100UserDriverReadApi(tPOCT6100_READ_PARAMS f_pReadParams)
 {
-        const unsigned int              addr  		=  f_pReadParams->ulReadAddress;
-	const struct echo_mod		*echo_mod 	= (struct echo_mod *)f_pReadParams->pProcessContext;
-	struct astribank_device 	*astribank 	= echo_mod->astribank;
+	const unsigned int              addr =  f_pReadParams->ulReadAddress;
+	const struct echo_mod		*echo_mod;
+	struct astribank_device 	*astribank;
+	int ret;
 
-	*f_pReadParams->pusReadData = echo_recv_data(astribank, addr);
+	echo_mod = (struct echo_mod *)f_pReadParams->pProcessContext;
+	astribank = echo_mod->astribank;
+	ret = echo_recv_data(astribank, addr);
+	if (ret < 0) {
+		ERR("echo_recv_data failed (%d)\n", ret);
+		return cOCT6100_ERR_FATAL_DRIVER_READ_API;
+	}
+	*f_pReadParams->pusReadData = ret;
 	return cOCT6100_ERR_OK;
 }
 
 UINT32 Oct6100UserDriverReadBurstApi(tPOCT6100_READ_BURST_PARAMS f_pBurstParams)
 {
-        unsigned int              	addr;
-        unsigned int              	len		= f_pBurstParams->ulReadLength;
-	const struct echo_mod		*echo_mod 	= (struct echo_mod *)f_pBurstParams->pProcessContext;
-	struct astribank_device 	*astribank 	= echo_mod->astribank;
+	unsigned int              	addr;
+	unsigned int              	len;
+	const struct echo_mod		*echo_mod;
+	struct astribank_device 	*astribank;
 	unsigned int 			i;
 
+	len = f_pBurstParams->ulReadLength;
+	echo_mod = (struct echo_mod *)f_pBurstParams->pProcessContext;
+	astribank = echo_mod->astribank;
 	for (i = 0;i < len; i++) {
+		unsigned int ret;
+
 		addr = f_pBurstParams->ulReadAddress + (i << 1);
-		f_pBurstParams->pusReadData[i] = echo_recv_data(astribank, addr);
+		ret = echo_recv_data(astribank, addr);
+		if (ret < 0) {
+			ERR("echo_recv_data failed (%d)\n", ret);
+			return cOCT6100_ERR_FATAL_DRIVER_READ_API;
+		}
+		f_pBurstParams->pusReadData[i] = ret;
 	}
 	return cOCT6100_ERR_OK;
 }
 
 inline int get_ver(struct astribank_device *astribank)
 {
- 
-	return  spi_send(astribank, 0, 0, 1, 1);
+	return spi_send(astribank, 0, 0, 1, 1);
 }
 
 //+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
@@ -511,18 +577,16 @@ UINT32 init_octasic(char *filename, struct astribank_device *astribank, int is_a
 	tOCT6100_CHANNEL_OPEN					ChannelOpen;
 	UINT32							ulChanHndl;
 
-	test_send(astribank);
+	if (test_send(astribank) < 0)
+		return cOCT6100_ERR_FATAL;
 	cpld_ver = get_ver(astribank);
-	INFO("%s [%s]: Check EC_CPLD version: %d\n",
-		xusb_devpath(astribank->xusb),
-		xusb_serial(astribank->xusb),
-		cpld_ver);
+	AB_INFO(astribank, "Check EC_CPLD version: %d\n", cpld_ver);
 	if (cpld_ver < 0)
-		return cpld_ver;
+		return cOCT6100_ERR_FATAL;
 	else if (cpld_ver == EC_VER_TEST) {
-		INFO("+---------------------------------------------------------+\n");
-		INFO("| WARNING: TEST HARDWARE IS ON THE BOARD INSTEAD OF EC!!! |\n");
-		INFO("+---------------------------------------------------------+\n");
+		AB_INFO(astribank, "+---------------------------------------------------------+\n");
+		AB_INFO(astribank, "| WARNING: TEST HARDWARE IS ON THE BOARD INSTEAD OF EC!!! |\n");
+		AB_INFO(astribank, "+---------------------------------------------------------+\n");
 		return cOCT6100_ERR_OK;
 	}
 
@@ -532,29 +596,31 @@ UINT32 init_octasic(char *filename, struct astribank_device *astribank, int is_a
 	/*	1) Configure and Open the OCT6100.			          */
 	/**************************************************************************/
 	/**************************************************************************/
-	
-        memset(&InstanceSize, 0, sizeof(tOCT6100_GET_INSTANCE_SIZE));
-        memset(&OpenChip, 0, sizeof(tOCT6100_CHIP_OPEN));
 
-        if (!(echo_mod = malloc(sizeof(struct echo_mod)))) {
-                ERR("cannot allocate memory for echo_mod\n");
-                return 1;
-        }
-                DBG("allocated mem for echo_mod\n");
+	memset(&InstanceSize, 0, sizeof(tOCT6100_GET_INSTANCE_SIZE));
+	memset(&OpenChip, 0, sizeof(tOCT6100_CHIP_OPEN));
 
-        memset(echo_mod, 0, sizeof(struct echo_mod));
+	echo_mod = malloc(sizeof(struct echo_mod));
+	if (!echo_mod) {
+		AB_ERR(astribank, "cannot allocate memory for echo_mod\n");
+		return cOCT6100_ERR_FATAL;
+	}
+	DBG("allocated mem for echo_mod\n");
+
+	memset(echo_mod, 0, sizeof(struct echo_mod));
 
 	/* Fill the OCT6100 Chip Open configuration structure with default values */
 
-	ulResult = Oct6100ChipOpenDef( &OpenChip );
+	ulResult = Oct6100ChipOpenDef(&OpenChip);
 	if (ulResult != cOCT6100_ERR_OK) {
-                ERR("Oct6100ChipOpenDef failed: result=%X\n", ulResult);
+		AB_ERR(astribank, "Oct6100ChipOpenDef failed: result=%X\n",
+			ulResult);
 		return ulResult;
 	}
 
 	OpenChip.pProcessContext      			= echo_mod;
 	/* Configure clocks */
-	
+
 	/* upclk oscillator is at 33.33 Mhz */
 	OpenChip.ulUpclkFreq 				= cOCT6100_UPCLK_FREQ_33_33_MHZ;
 
@@ -583,70 +649,75 @@ UINT32 init_octasic(char *filename, struct astribank_device *astribank, int is_a
 
 	/* External Memory Settings: Use DDR memory*/
 	OpenChip.ulMemoryType				= cOCT6100_MEM_TYPE_DDR;
-	
+
 	OpenChip.ulNumMemoryChips			= 1;
 	OpenChip.ulMemoryChipSize			= cOCT6100_MEMORY_CHIP_SIZE_32MB;
 
 
 	/* Load the image file */
 	ulResult = load_file(	filename,
-				&pbyImageData,
-				&ulImageByteSize );
+			&pbyImageData,
+			&ulImageByteSize);
 
-	if (pbyImageData == NULL || ulImageByteSize == 0){
-		ERR("Bad pbyImageData or ulImageByteSize\n");
-		return 1;
-	}
-	if ( ulResult != 0 ) {
-		ERR("Failed load_file %s (%08X)\n", filename, ulResult);
+	if (ulResult != 0) {
+		AB_ERR(astribank, "Failed load_file %s (%08X)\n", filename, ulResult);
 		return ulResult;
 	}
+	if (pbyImageData == NULL || ulImageByteSize == 0){
+		AB_ERR(astribank, "Bad pbyImageData or ulImageByteSize\n");
+		return cOCT6100_ERR_FATAL;
+	}
 
 	/* Assign the image file.*/
 	OpenChip.pbyImageFile				= pbyImageData;
 	OpenChip.ulImageSize				= ulImageByteSize;
 
-        /* Inserting default values into tOCT6100_GET_INSTANCE_SIZE structure parameters. */
-        Oct6100GetInstanceSizeDef ( &InstanceSize );
+	/*
+	 * Inserting default values into tOCT6100_GET_INSTANCE_SIZE
+	 * structure parameters.
+	 */
+	Oct6100GetInstanceSizeDef(&InstanceSize);
 
-        /* Get the size of the OCT6100 instance structure. */
-        ulResult = Oct6100GetInstanceSize(&OpenChip, &InstanceSize );
-        if (ulResult != cOCT6100_ERR_OK)
-        {
-                ERR("Oct6100GetInstanceSize failed (%08X)\n", ulResult);
-                return ulResult;
-        }
+	/* Get the size of the OCT6100 instance structure. */
+	ulResult = Oct6100GetInstanceSize(&OpenChip, &InstanceSize);
+	if (ulResult != cOCT6100_ERR_OK) {
+		AB_ERR(astribank, "Oct6100GetInstanceSize failed (%08X)\n",
+			ulResult);
+		return ulResult;
+	}
 
-        pApiInstance = malloc(InstanceSize.ulApiInstanceSize);
+	pApiInstance = malloc(InstanceSize.ulApiInstanceSize);
 	echo_mod->pApiInstance 				= pApiInstance;
 	echo_mod->astribank 				= astribank;
 
-        if (!pApiInstance) {
-                ERR("Out of memory (can't allocate %d bytes)!\n", InstanceSize.ulApiInstanceSize);
-                return 1;
-        }
+	if (!pApiInstance) {
+		AB_ERR(astribank, "Out of memory (can't allocate %d bytes)!\n",
+			InstanceSize.ulApiInstanceSize);
+		return cOCT6100_ERR_FATAL;
+	}
 
 	/* Perform actual open of chip */
 	ulResult = Oct6100ChipOpen(pApiInstance, &OpenChip);
 	if (ulResult != cOCT6100_ERR_OK) {
-                ERR("Oct6100ChipOpen failed: result=%X\n", ulResult);
+		AB_ERR(astribank, "Oct6100ChipOpen failed: result=%X\n",
+			ulResult);
 		return ulResult;
 	}
 	DBG("%s: OCT6100 is open\n", __func__);
 
 	/* Free the image file data  */
-	free( pbyImageData );
-	
+	free(pbyImageData);
+
 	/**************************************************************************/
 	/**************************************************************************/
 	/*	2) Open channels in echo cancellation mode.                       */
 	/**************************************************************************/
 	/**************************************************************************/
 
-	for( nChan = 0; nChan < ECHO_MAX_CHANS; nChan++ ) {
+	for (nChan = 0; nChan < ECHO_MAX_CHANS; nChan++) {
 		nSlot 						= nChan;
 		/* open a channel.*/
-		Oct6100ChannelOpenDef( &ChannelOpen );
+		Oct6100ChannelOpenDef(&ChannelOpen);
 
 		/* Assign the handle memory.*/
 		ChannelOpen.pulChannelHndl = &ulChanHndl;
@@ -668,7 +739,7 @@ UINT32 init_octasic(char *filename, struct astribank_device *astribank, int is_a
 		ChannelOpen.TdmConfig.ulRoutPcmLaw		= pcmLaw;
 		ChannelOpen.TdmConfig.ulRoutStream		= ECHO_ROUT_STREAM;
 		ChannelOpen.TdmConfig.ulRoutTimeslot		= nSlot;
-		
+
 		ChannelOpen.TdmConfig.ulSoutPcmLaw		= pcmLaw;
 		ChannelOpen.TdmConfig.ulSoutStream		= ECHO_SOUT_STREAM;
 		ChannelOpen.TdmConfig.ulSoutTimeslot		= nSlot;
@@ -679,15 +750,15 @@ UINT32 init_octasic(char *filename, struct astribank_device *astribank, int is_a
 		ChannelOpen.VqeConfig.fSinDcOffsetRemoval	= TRUE;
 
 		ChannelOpen.VqeConfig.ulComfortNoiseMode	= cOCT6100_COMFORT_NOISE_NORMAL;	
-							/*        cOCT6100_COMFORT_NOISE_NORMAL
-								  cOCT6100_COMFORT_NOISE_EXTENDED,
-								  cOCT6100_COMFORT_NOISE_OFF,
-								  cOCT6100_COMFORT_NOISE_FAST_LATCH 
-							 */
+		/*        cOCT6100_COMFORT_NOISE_NORMAL
+			  cOCT6100_COMFORT_NOISE_EXTENDED,
+			  cOCT6100_COMFORT_NOISE_OFF,
+			  cOCT6100_COMFORT_NOISE_FAST_LATCH
+		 */
 		ulResult = Oct6100ChannelOpen(	pApiInstance,
-						&ChannelOpen );
+				&ChannelOpen);
 		if (ulResult != cOCT6100_ERR_OK) {
-			ERR("Found error on chan %d\n", nChan);
+			AB_ERR(astribank, "Found error on chan %d\n", nChan);
 			return ulResult;
 		}
 	}
@@ -698,10 +769,10 @@ UINT32 init_octasic(char *filename, struct astribank_device *astribank, int is_a
 	/**************************************************************************/
 	/**************************************************************************/
 
-	for( nChan = 8; nChan < 32; nChan++ ) {
+	for (nChan = 8; nChan < 32; nChan++) {
 		nSlot 						= (nChan >> 3) * 32 + (nChan & 0x07);
 		/* open a channel.*/
-		Oct6100ChannelOpenDef( &ChannelOpen );
+		Oct6100ChannelOpenDef(&ChannelOpen);
 
 		/* Assign the handle memory.*/
 		ChannelOpen.pulChannelHndl = &ulChanHndl;
@@ -718,7 +789,7 @@ UINT32 init_octasic(char *filename, struct astribank_device *astribank, int is_a
 
 		ChannelOpen.TdmConfig.ulRoutStream		= ECHO_ROUT_STREAM2;
 		ChannelOpen.TdmConfig.ulRoutTimeslot		= nSlot;
-		
+
 		ChannelOpen.TdmConfig.ulSoutStream		= ECHO_SOUT_STREAM2;
 		ChannelOpen.TdmConfig.ulSoutTimeslot		= nSlot;
 
@@ -728,15 +799,15 @@ UINT32 init_octasic(char *filename, struct astribank_device *astribank, int is_a
 		ChannelOpen.VqeConfig.fSinDcOffsetRemoval	= TRUE;
 
 		ChannelOpen.VqeConfig.ulComfortNoiseMode	= cOCT6100_COMFORT_NOISE_NORMAL;	
-							/*        cOCT6100_COMFORT_NOISE_NORMAL
-								  cOCT6100_COMFORT_NOISE_EXTENDED,
-								  cOCT6100_COMFORT_NOISE_OFF,
-								  cOCT6100_COMFORT_NOISE_FAST_LATCH 
-							 */
+		/*        cOCT6100_COMFORT_NOISE_NORMAL
+			  cOCT6100_COMFORT_NOISE_EXTENDED,
+			  cOCT6100_COMFORT_NOISE_OFF,
+			  cOCT6100_COMFORT_NOISE_FAST_LATCH
+		 */
 		ulResult = Oct6100ChannelOpen(	pApiInstance,
-						&ChannelOpen );
+				&ChannelOpen);
 		if (ulResult != cOCT6100_ERR_OK) {
-			ERR("Found error on chan %d\n", nChan);
+			AB_ERR(astribank, "Found error on chan %d\n", nChan);
 			return ulResult;
 		}
 	}
@@ -752,24 +823,21 @@ UINT32 init_octasic(char *filename, struct astribank_device *astribank, int is_a
 
 int load_echo(struct astribank_device *astribank, char *filename, int is_alaw)
 {
-	int		iLen;
 	int		ret;
-	unsigned char	*pbyFileData = NULL; 
-	const char	*devstr;
+	UINT32		octasic_status;
 
-	devstr = xusb_devpath(astribank->xusb);
-	INFO("%s [%s]: Loading ECHOCAN Firmware: %s (%s)\n",
-		devstr, xusb_serial(astribank->xusb), filename,
-		(is_alaw) ? "alaw" : "ulaw");
+	AB_INFO(astribank, "Loading ECHOCAN Firmware: %s (%s)\n",
+		filename, (is_alaw) ? "alaw" : "ulaw");
 	usb_buffer_init(astribank, &usb_buffer);
-	ret = init_octasic(filename, astribank, is_alaw);
-	if (ret) {
-		ERR("ECHO %s burning failed (%08X)\n", filename, ret);
+	octasic_status = init_octasic(filename, astribank, is_alaw);
+	if (octasic_status != cOCT6100_ERR_OK) {
+		AB_ERR(astribank, "ECHO %s burning failed (%08X)\n",
+			filename, octasic_status);
 		return -ENODEV;
 	}
 	ret = usb_buffer_flush(astribank, &usb_buffer);
 	if (ret < 0) {
-		ERR("ECHO %s buffer flush failed (%d)\n", filename, ret);
+		AB_ERR(astribank, "ECHO %s buffer flush failed (%d)\n", filename, ret);
 		return -ENODEV;
 	}
 	usb_buffer_showstatistics(astribank, &usb_buffer);
